package com.shiyanlou.core.validate;

import com.shiyanlou.core.properties.SecurityProperties;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.web.authentication.AuthenticationFailureHandler;
import org.springframework.social.connect.web.HttpSessionSessionStrategy;
import org.springframework.social.connect.web.SessionStrategy;
import org.springframework.stereotype.Component;
import org.springframework.util.AntPathMatcher;
import org.springframework.web.bind.ServletRequestBindingException;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.context.request.ServletWebRequest;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * 验证码验证过滤器
 * OncePerRequestFilter spring提供的，保证在一个请求中只会被调用一次
 */
@Component("validateCodeFilter")
public class ValidateCodeFilter extends OncePerRequestFilter {
    // 在初始化本类的地方进行注入
    // 一般在配置security http的地方进行添加过滤器
    private AuthenticationFailureHandler failureHandler;
    private SessionStrategy sessionStrategy = new HttpSessionSessionStrategy();

    @Autowired
    private SecurityProperties securityProperties;


    //存放所有需要校验验证码的url
    private static List<String> urlList = new ArrayList();

    //验证请求url与配置的url是否匹配的工具类
    private AntPathMatcher pathMatcher = new AntPathMatcher();

    /*
     * 功能描述：初始化要拦截的url配置信息
     */
    @Override
    public void afterPropertiesSet() throws ServletException {
        super.afterPropertiesSet();

        //手机验证码需要拦截的路径
        for (String url : securityProperties.getCode().getInterceptUrls()) {
            addUrlToList(url);
        }
    }

    /*
     * 功能描述：将系统中配置的需要校验验证码的URL根据校验的类型放入list【一项配置成多个时，用“,”隔开：a[0]:/aaa,/bbb】
     * 返回值：void
     */
    protected void addUrlToList(String urlString) {
        if (StringUtils.isNotBlank(urlString)) {
            String[] urls = StringUtils.splitByWholeSeparatorPreserveAllTokens(urlString, ",");
            for (String url : urls) {
                urlList.add(url);
            }
        }
    }

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain) throws ServletException, IOException {

        //这里限制不能是get请求方式
        if (!StringUtils.equalsIgnoreCase(request.getMethod(), "get")) {
            // 循环遍历：
            for (String url : urlList) {
                //支持通配符
                if (pathMatcher.match(url, request.getRequestURI())) {
                    try {
                        validate(request);//验证
                    } catch (ValidateCodeException e) {
                        failureHandler.onAuthenticationFailure(request, response, e);
                        return;
                    }
                    break;
                }
            }
        }

        filterChain.doFilter(request, response);
    }


    private void validate(HttpServletRequest request) throws ServletRequestBindingException {
        // 拿到之前存储的smsCode信息
        ServletWebRequest swr = new ServletWebRequest(request);
        SmsCode codeInSession = (SmsCode) sessionStrategy.getAttribute(swr, ValidateCodeController.SESSION_KEY);

        String codeInRequest = ServletRequestUtils.getStringParameter(request, "smsCode");

        if (StringUtils.isBlank(codeInRequest)) {
            throw new ValidateCodeException("验证码的值不能为空");
        }
        if (codeInSession == null) {
            throw new ValidateCodeException("验证码不存在");
        }
        if (codeInSession.isExpried()) {
            sessionStrategy.removeAttribute(swr, ValidateCodeController.SESSION_KEY);
            throw new ValidateCodeException("验证码已过期");
        }
        if (!StringUtils.equals(codeInSession.getCode(), codeInRequest)) {
            throw new ValidateCodeException("验证码不匹配");
        }
        //移除验证码
        sessionStrategy.removeAttribute(swr, ValidateCodeController.SESSION_KEY);
    }

    public AuthenticationFailureHandler getFailureHandler() {
        return failureHandler;
    }
    public void setFailureHandler(AuthenticationFailureHandler failureHandler) {
        this.failureHandler = failureHandler;
    }
}